<?php
require_once(_PS_MODULE_DIR_.'pss_timemsgpro/classes/PssTimeMsgProUtils.php');

if (!defined('_PSS_TIMEMSGPRO_DIR_')) define('_PSS_TIMEMSGPRO_DIR_', _PS_MODULE_DIR_.'pss_timemsgpro/');	// base dir for current module

class PssTimeMsgPro extends ObjectModel
{
	public 		$id;
	
	// common fields ----------------------
	/** @var string message name */
	public 		$name;
	
	/** @var string hook for this message */
	public 		$hook_name;
	
	/** @var string template name to display this message */
	public 		$template_name;
	
	/** @var string CSS name used to display this message */
	public 		$css_name;
	
	/** @var string user_class to be called by hook */
	public 		$user_class;
	
	/** @var int Active */
	public 		$active;
	
	/** @var int Customer is logged restriction - 0 means no restriction - non zero positive int means restriction */
	public 		$is_logged;
	
	/** @var int Customer group restriction - 0 means no restriction - non zero positive int means restriction */
	public 		$id_group;
	
	/** @var int Category restriction - 0 means no restriction - non zero positive int means restriction */
	public 		$id_category;
	
	/** @var int Product restriction - 0 means no restriction - non zero positive int means restriction */
	public 		$id_product;
	
	/** @var String - field to store any other filter rule */
	public 		$other_rule;
	
	/** @var String - script name list not to display the message */
	public 		$exceptions;
	
	/** @var int Order */
	public 		$order;
	
	/** @var Date from rule display */
	public 		$from_date;
	
	/** @var Date to rule display */
	public 		$to_date;
	
	/** @var String : cron expression to start a recurrent display */
	public		$cron_expression;
	
	/** @var int : duration of a display that starts according to a cron expression */
	public		$duration;
	
	// lang fields ----------------------
	/** @var string message itself */
	public 		$msg;
	
	
 	protected 	$fieldsRequired = array('name');
 	protected 	$fieldsSize = array('name' => 255);
 	protected 	$fieldsValidate = array('name' => 'isGenericName', 
										'hook_name' => 'isHookName', 
// @change for Prestashop 1.5 compatibility
// Change 1.5-No need of this control (not compatible with PS 1.5) - check is done in postProcess function of module tab
//										'template_name' => 'isTplFileName', 
										'css_name' => 'isFileName', 
										'active' => 'isBool', 
//										'is_logged' => 'isBool', 
										'id_group' => 'isInt', 
										'id_category' => 'isInt', 
										'id_product' => 'isInt', 
										'order' => 'isInt', 
										'from_date' => 'isDate', 
										'to_date' => 'isDate', 
										'duration' => 'isInt');
	
// 	protected 	$fieldsRequiredLang = array('msg');
 	protected 	$fieldsRequiredLang = array();
 	protected 	$fieldsSizeLang = array();
 	protected 	$fieldsValidateLang = array('msg' => 'isCleanHtml');
	
	protected 	$table = 'prestascope_timemsgpro';
	protected 	$identifier = 'id_msg';

	public function getFields()
	{
		parent::validateFields();
		
		$fields['name'] = pSQL($this->name);
		$fields['hook_name'] = pSQL($this->hook_name);
		$fields['template_name'] = pSQL($this->template_name);
		$fields['css_name'] = pSQL($this->css_name);
		$fields['user_class'] = pSQL($this->user_class);
		$fields['active'] = (int)($this->active);
		$fields['is_logged'] = (int)($this->is_logged);
		$fields['id_group'] = (int)($this->id_group);
		$fields['id_category'] = (int)($this->id_category);
		$fields['id_product'] = (int)($this->id_product);
		$fields['other_rule'] = pSQL($this->other_rule);
		$fields['exceptions'] = pSQL(str_replace(" ", "", $this->exceptions));
		$fields['order'] = (int)($this->order);
		$fields['from_date'] = pSQL($this->from_date);
		$fields['to_date'] = pSQL($this->to_date);
		$fields['cron_expression'] = pSQL($this->cron_expression);
		$fields['duration'] = pSQL($this->duration);
		
		return $fields;
	}
	/**
	 * Check then return multilingual fields for database interaction
	 *
	 * @return array Multilingual fields
	 */
	public function getTranslationsFieldsChild()
	{
		parent::validateFieldsLang();
		return $this->getTranslationsFields(array('msg'=>true));
	}
	/**
	 * Prepare multilingual fields for database insertion(COPIED FROM PRESTASHOP ObjectModel.php - V1.3.7 but hacked to deal with HTML content)
	 *
	 * @param array $fieldsArray Multilingual fields to prepare
	 * return array Prepared fields for database insertion
	 */
	protected function getTranslationsFields($fieldsArray)
	{
		/* WARNING : Product do not use this function, so do not forget to report any modification if necessary */
	 	if (!Validate::isTableOrIdentifier($this->identifier))
	 		die(Tools::displayError());

		$fields = array();
		// @change for Prestashop 1.5 compatibility
		if (PssTimeMsgProUtils::isPs15x())
			$languages = Language::getLanguages(false);
		else
			$languages = Language::getLanguages();
		$defaultLanguage = Configuration::get('PS_LANG_DEFAULT');
		foreach ($languages as $language)
		{
			$fields[$language['id_lang']]['id_lang'] = $language['id_lang'];
			$fields[$language['id_lang']][$this->identifier] = intval($this->id);
			foreach ($fieldsArray as $field=>$isHtml)
			{
	 			/* Check fields validity */
			 	if (!Validate::isTableOrIdentifier($field))
	 				die(Tools::displayError());

				echo 'Language '.$language['id_lang'].' / check field '.$field.' that is html '.$isHtml.'<br />';
				/* Copy the field, or the default language field if it's both required and empty */
				if (isset($this->{$field}[$language['id_lang']]) AND !Tools::isEmpty($this->{$field}[$language['id_lang']]))
				{
					// hack to deal with HTML datas
					if (!$isHtml)
						$fields[$language['id_lang']][$field] = pSQL($this->{$field}[$language['id_lang']]);
					else
					{
						$fields[$language['id_lang']][$field] = $this->{$field}[$language['id_lang']];
						// deal with ' that could come from tinymce (could be found in inline style="background:url('...')")
						$fields[$language['id_lang']][$field]  = str_replace("'", "&#39;", $fields[$language['id_lang']][$field]);
					}
				}
				elseif (in_array($field, $this->fieldsRequiredLang))
				{
					// hack to deal with HTML datas
					if (!$isHtml)
						$fields[$language['id_lang']][$field] = pSQL($this->{$field}[$defaultLanguage]);
					else
					{
						$fields[$language['id_lang']][$field] = $this->{$field}[$defaultLanguage];
						// deal with ' that could come from tinymce (could be found in inline style="background:url('...')")
						$fields[$language['id_lang']][$field]  = str_replace("'", "&#39;", $fields[$language['id_lang']][$field]);
					}
				}
				else
					$fields[$language['id_lang']][$field] = '';
			}
		}

		return $fields;
	}
	/** 
	 * Overrides parent function because we need to use nullValues and also to update the hooks this module is associated to
	 */
	public function add($autodate = true, $nullValues = false)
	{
		// use NULL
		$success = parent::add(false, true);
		
		return $success;
	}
	/** 
	 * Overrides parent function because we need to use nullValues and also to update the hooks this module is associated to
	 */
	public function update($nullValues = false)
	{
		// use NULL
		$success = parent::update(true);

		return $success;
	}
	/**
	 * Overrides parent function because we need to update the hooks this module is associated to
	 */
	public function delete()
	{
		// use NULL
		$success = parent::delete();

		return $success;
	}
	/**
	 * Update the hook list for the module
	 */
	public static function updateHooksForModule() 
	{
		PssTimeMsgProUtils::addLog("... Call to updateHooksForModule");
		// get all objects
		$objs = PssTimeMsgPro::getAll();
		
		// get the module object
		$module = Module::getInstanceByName('pss_timemsgpro');
		
		// load a map between all hook names and their id
		$tempHookObjs = Db::getInstance()->ExecuteS('
		SELECT h.`id_hook`, h.`name` 
		FROM `'._DB_PREFIX_.'hook` h');
		$hookIdByName = array();
		foreach($tempHookObjs as $obj)
			$hookIdByName[$obj['name']] = $obj['id_hook'];
		
		// build the list of all hook names the module must be associated to
		$expectedHookNames = array();
		// always add header
		$expectedHookNames[] = 'header';
		foreach($objs as $obj)
			$expectedHookNames[] = $obj['hook_name'];
		$expectedHookNames = array_unique($expectedHookNames);
//		echo 'Expected hooks : ------------------<br />';
//		print_r($expectedHookNames);
//		echo '<br />';
		
		// get the list of all hooks the module is currently associated to
		$currentHookObjs = Db::getInstance()->ExecuteS('
		SELECT h.`name`
		FROM `'._DB_PREFIX_.'hook_module` hm
		LEFT JOIN `'._DB_PREFIX_.'hook` h ON (hm.id_hook = h.id_hook)
		WHERE hm.id_module = '.intval($module->id));
		$currentHookNames = array();
		foreach ($currentHookObjs as $obj)
			$currentHookNames[] = $obj['name'];
		$currentHookNames = array_unique($currentHookNames);
//		echo 'Current hooks : ------------------<br />';
//		print_r($currentHookNames);
//		echo '<br />';
		
		// compute the list of hooks the module is associated and should not anymore (delete the assocation)
		$toDelete = array_diff($currentHookNames, $expectedHookNames);
		if (!empty($toDelete))
		{
			foreach($toDelete as $hookName)
			{
//				echo 'Unregister hook '.$hookName.'<br />';
				$module->unregisterHook($hookIdByName[$hookName]);
			}
		}
		
		// compute the list of hooks the module is not associated and should be (create the assocation)
		$toCreate = array_diff($expectedHookNames, $currentHookNames);
		if (!empty($toCreate))
		{
			foreach($toCreate as $hookName)
			{
//				echo 'Register hook '.$hookName.'<br />';
				$module->registerHook($hookName);
			}
		}
		
	}
	
	/**
	  * Return all available objects
	  *
	  * @param integer $id_lang : non null Language ID to get object details only for this language
	  * @param integer $active : true to get only active objects
	  * @return array objects
	  */
	public static function getAll($id_lang=null, $active=false)
	{
		$query = '
			SELECT *
			FROM `'._DB_PREFIX_.'prestascope_timemsgpro` s
			LEFT JOIN `'._DB_PREFIX_.'prestascope_timemsgpro_lang` sl ON s.`id_msg` = sl.`id_msg` WHERE 1=1';
		if ($id_lang!=null)
			$query .= ' AND sl.`id_lang` = '.(int)($id_lang);
		if ($active)
			$query .= ' AND s.`active` = 1';
		$objects = Db::getInstance()->ExecuteS($query);
		return $objects;
	}
	/**
	 * Return all Time messages for a hook ok to be displayed now
	 */
	public static function getByHook($hook_name, $params, $smartyV2, $id_lang=null)
	{
		global $cookie, $smarty;
		
		/*
		 SQL Tactic to earn time and save global module performances : build a query like this one :
			SELECT * FROM `ps_prestascope_timemsgpro` s 
				LEFT JOIN `ps_prestascope_timemsgpro_lang` sl ON s.`id_msg` = sl.`id_msg` 
			WHERE 1=1 AND 
				s.`active` = 1 
					AND 
				sl.`id_lang` = 2 
					AND 
				s.`hook_name` = 'leftColumn' 
					AND 
				IFNULL(s.`from_date`, '2000-01-01') <= CURRENT_DATE() 
					AND 
				IFNULL(s.`to_date`, '3000-01-01') >= CURRENT_DATE() 

		 that embed all filters :
			 > active flag
			 > hook name
			 > dates
		 */
		
//		echo 'HOOK : '.$hook_name.' : <br />';
//		print_r($param);
//		echo '--------------------<br />';
		
		$query = '
			SELECT *
			FROM `'._DB_PREFIX_.'prestascope_timemsgpro` s
			LEFT JOIN `'._DB_PREFIX_.'prestascope_timemsgpro_lang` sl ON s.`id_msg` = sl.`id_msg` WHERE 1=1';
		// only active mesages
		$query .= " AND s.`active` = 1";
		// retrict on lang
		if ($id_lang!=null)
			$query .= ' AND sl.`id_lang` = '.(int)($id_lang);
		// check the hook
		$query .= " AND s.`hook_name` = '".$hook_name."'";
		// check the dates
		$query .= " AND IFNULL(s.`from_date`, '2000-01-01') <= CURRENT_DATE()";
		$query .= " AND IFNULL(s.`to_date`, '3000-01-01') >= CURRENT_DATE()";
		
		// get objects
//		echo $query.'<br />';
		$objects = Db::getInstance()->ExecuteS($query);		
		
		return $objects;
	}
}
