<?php

/**
  * Prestascope Tools class
  * @category classes
  *
  * @author Sylvain Caillet <s.caillet@free.com>
  * @copyright Sylvain Caillet
  * @license - see file
  * History :
  *
  */
if (!defined('_PSS_LOG_LIMIT_')) define('_PSS_LOG_LIMIT_', '5242880');				// Log files limit : 5 Mo (Max 4 files named : log/pss.0.log, log/pss.1.log, log/pss.2.log, log/pss.3.log
if (!defined('_PSS_TIMEMSGPRO_DIR_')) define('_PSS_TIMEMSGPRO_DIR_', _PS_MODULE_DIR_.'pss_timemsgpro/');	// base dir for current module
  
class PssTimeMsgProUtils
{
	/*
	 * Add a line to import log file
	 */
	public static function addLog($txt) {
		$file = _PSS_TIMEMSGPRO_DIR_ . 'log/pss.0.log';
		// append date
		$log = "";
		$log .= "(" . date("d-m-Y H:i:s") . ") ";
		$log .= $txt;
		$log .= "\n";

		$handle = fopen($file, "a");
		fwrite($handle, utf8_encode($log));
		fclose($handle);
	}

	/*
	 * Check if log size is bigger than a limit. If it's the case, then rename all logs (rotate names)
	 * and may be delete the oldest one.
	 */
	public static function manageLogSize() {
		// current log size
		$file = _PSS_TIMEMSGPRO_DIR_ . 'log/pss.0.log';
		if (@filesize($file)>_PSS_LOG_LIMIT_) {
			// unlink the oldest
			@unlink(_PSS_TIMEMSGPRO_DIR_ . 'log/pss.3.log');

			// rename others
			@copy (_PSS_TIMEMSGPRO_DIR_ . 'log/pss.2.log', _PSS_TIMEMSGPRO_DIR_ . 'log/pss.3.log');
			@copy (_PSS_TIMEMSGPRO_DIR_ . 'log/pss.1.log', _PSS_TIMEMSGPRO_DIR_ . 'log/pss.2.log');
			@copy (_PSS_TIMEMSGPRO_DIR_ . 'log/pss.0.log', _PSS_TIMEMSGPRO_DIR_ . 'log/pss.1.log');

			// unlink current
			@unlink(_PSS_TIMEMSGPRO_DIR_ . 'log/pss.0.log');
		}
	}
	/*
	 * Remove all log files in input dir.
	 */
	public static function removeLogs() {
		@unlink (_PSS_TIMEMSGPRO_DIR_ . 'log/pss.0.log');
		@unlink (_PSS_TIMEMSGPRO_DIR_ . 'log/pss.1.log');
		@unlink (_PSS_TIMEMSGPRO_DIR_ . 'log/pss.2.log');
		@unlink (_PSS_TIMEMSGPRO_DIR_ . 'log/pss.3.log');
	}
	/**
	 * Check if a table exists in a db
	 */
	public static function mysql_table_exists($table , $db) 
	{	
		$tables=mysql_list_tables($db); 
		while (list($temp)=mysql_fetch_array($tables)) 
		{
			if($temp == $table) 
				return 1; 
		}
		return 0; 
	}
	/**
	 * Capture a var dump into a string
	 */
	public static function captureVarDump($var)
	{
		ob_start();
		if (!is_array($var))
			var_dump($var);
		else
			print_r($var);	// take less characters to display an array
		$out = ob_get_contents();
		ob_end_clean();
		return $out;
	}
	/**
	 * Capture a user friendly var dump into a string
	 */
	public static function captureBestVarDump($var)
	{
		$o = "";
		if (is_array($var))
		{
			$o.="array:\n";
			foreach($var as $key=>$item)
				$o.="\t".$key." => ".(is_array($item)?"array":is_object($item)?"object":$item)."\n";
		}
		else if (is_object($var))
		{
			$o.="object";
		}
		else
		{
			$o.=$var;
		}
		return $o;
	}
	/**
	 * Prepare a value to be exported in a CSV file
	 */
	public static function prepareValueForCsv($value) 
	{
		// work on a clone
		$val = $value;
		// replace all carriage return chars
		$val = str_replace(CHR(13).CHR(10)," ",$val); 
		
		return $val;
	}
	/**
	 * Replace the last occurence of s string in a string
	 */
	public function str_lreplace($search, $replace, $subject)
	{
		$pos = strrpos($subject, $search);

		if($pos !== false)
		{
			$subject = substr_replace($subject, $replace, $pos, strlen($search));
		}

		return $subject;
	}
	/**
	 * Build an URL to display the admin module position screen.
	 * @param currentIndex:current admin index used to get the root admin URL
	 * @param hook_name : optional hook name to activate a HREF on the hook group
	 */
	public static function getAdminModulePositionUrl($currentIndex, $hook_name=null)
	{
		global $cookie;
		
		// extract from current index (http://localhost/ps137/admin137/index.php?tab=adminpss_timemsgpro&token=e18cb240bade142a949395b9821d036e) the admin root url
		$rootAdminUrl = substr($currentIndex, 0, strpos($currentIndex, '?'));
		
		// get the AdminModulesPositions id
		$positionTabId = Tab::getIdFromClassName('AdminModulesPositions');
		
		// get the token value
		$positionToken = Tools::getAdminToken('AdminModulesPositions'.intval($positionTabId).intval($cookie->id_employee));
		
		// finally the URL
		$positionUrl = $rootAdminUrl.'?tab=AdminModulesPositions&token='.$positionToken.($hook_name?'#'.$hook_name:'');
	
		return $positionUrl;
	}
	/**
	 * Prepare a value to be used in Javascript
	 */
	public static function prepareValueForJs($value)
	{
		if (self::checkPsVersion('1.4.0') || !self::isPs14x())
			return utf8_encode(str_replace('"', '\"', html_entity_decode($value)));
		else
			return str_replace('"', '\"', html_entity_decode($value));
	}
	/**
	  * load product's categories
	  */
	public static function getProductCategories($id_product)
	{
		$result = array();
		$query = '
			SELECT id_category
			FROM `'._DB_PREFIX_.'category_product` cp
			WHERE id_product='.$id_product;
		$objects = Db::getInstance()->ExecuteS($query);
		foreach($objects as $object)
			$result[] = $object['id_category'];
		return $result;
	}

	/**
	 * Check if installed Prestashop is a 1.2.x version
	 */
	public static function isPs12x() 
	{
		return self::checkPsVersion('1.2');
	}
	/**
	 * Check if installed Prestashop is a 1.3.x version
	 */
	public static function isPs13x() 
	{
		return self::checkPsVersion('1.3');
	}
	/**
	 * Check if installed Prestashop is a 1.4.x version
	 */
	public static function isPs14x() 
	{
		return self::checkPsVersion('1.4');
	}
	// @change for Prestashop 1.5 compatibility
	/**
	 * Check if installed Prestashop is a 1.5.x version
	 */
	public static function isPs15x() 
	{
		return self::checkPsVersion('1.5');
	}
	/**
	 * Check if installed Prestashop match an input radix
	 */
	public static function checkPsVersion($radixVersion) 
	{
		// get PS version
		$psVersion = _PS_VERSION_;
		if ($psVersion==null)
			return false;
		
		// look for version like 1.3.7.0
		$subVersions = explode('.', $psVersion);
		$searchVersions = explode('.', $radixVersion);
		
		for ($i=0;$i<count($searchVersions);$i++) 
		{
			// compare each sub part of version
			if ($subVersions[$i] !== $searchVersions[$i]) 
				return false;
		}
		return true;
	}
	
}