<?php
/**
 * Display custom HTML texts to users in different places of a shop only between dates.
 *
 * See LICENCE.txt for terms of use
 * History :
 * 	@version 1.5 (2012-11-10) : 
 *		Compatible with Prestashop 1.5
 * 		Doesn't support the multi-shop feature
 *		Custom class must now return a boolean to let the template display or not
 * 	@version 1.4 : 
 *		1st commercial version
 *		
 */
if (!defined('_PS_VERSION_'))
  exit;
  
require_once(_PS_MODULE_DIR_.'pss_timemsgpro/classes/PssTimeMsgPro.php');
require_once(_PS_MODULE_DIR_.'pss_timemsgpro/classes/PssTimeMsgProUtils.php');
require_once(_PS_MODULE_DIR_.'pss_timemsgpro/classes/PssTimeMsgProAbstractUserClass.php');

if (!defined('_PSS_TIMEMSGPRO_LICENCE_')) define('_PSS_TIMEMSGPRO_LICENCE_', 'BASIC');
//if (!defined('_PSS_TIMEMSGPRO_LICENCE_')) define('_PSS_TIMEMSGPRO_LICENCE_', 'PRO');

class pss_timemsgpro extends Module
{
    private $_postErrors = array();
	private $_tabClass = 'adminpss_timemsgpro';
	private $_moduleNames;
	
	private $tabParentName = 'AdminPreferences';

	private $smartyV2;	// load the force smarty V2 preference
	
	// config default set
	public static $_config = array(
		'PRESTASCOPE_TIMEMSG_VERSION' => '1.5',	// Change the history in adminpss_timemsgpro.php
	);

    function __construct()
    {
        $this->name = 'pss_timemsgpro';
		// some changes between 1.3.x and 1.4.x Prestashop versions
		if (PssTimeMsgProUtils::isPs12x() || PssTimeMsgProUtils::isPs13x()) 
		{
			$this->tab = 'Prestascope';
		} 
		else if (PssTimeMsgProUtils::isPs14x()) 
		{
			$this->tab = 'front_office_features';
			$this->author = 'PrestaScope';
		}
		// @change for Prestashop 1.5 compatibility
		else if (PssTimeMsgProUtils::isPs15x())
		{
			$this->tab = 'front_office_features';
			$this->author = 'PrestaScope';
			// set the version compliancy
			$this->ps_versions_compliancy = array('min' => '1.2', 'max' => '1.6');	// 1.2 included / 1.6 excluded
		}
			
		$installedVersion = Configuration::get('PRESTASCOPE_TIMEMSG_VERSION');
		if (!$installedVersion) // not installed yet
			$this->version = self::$_config['PRESTASCOPE_TIMEMSG_VERSION'];
		else
			$this->version = $installedVersion;
			
		// define names of module displayed in tab
		if (_PSS_TIMEMSGPRO_LICENCE_==='BASIC')
			$this->_moduleNames = array('en'=>'PSS/Time messages', 'fr'=>'PSS/Messages temporels', 'es'=>'PSS/Mensajes temporales');
		else
			$this->_moduleNames = array('en'=>'PSS/Shop scheduler', 'fr'=>'PSS/Shop programmateur', 'es'=>'PSS/Shop planificador');

        parent::__construct();

		// load the force Smarty V2 pref
		$this->smartyV2 = Configuration::get('PS_FORCE_SMARTY_2');
		
		if (_PSS_TIMEMSGPRO_LICENCE_==='BASIC')
			$this->displayName = $this->l('PSS/Time messages');
		else
			$this->displayName = $this->l('PSS/Shop scheduler');
		$buf = '<style type="text/css">';
		$buf .= 'a.descriptionLink {color:blue;text-decoration:underline;} a.descriptionLink:hover{color:red;text-decoration:none;}';
		$buf .= 'div.pssDescriptionDiv {background:#6a6a6a; color:white; padding:3px; margin-top:2px;}';
		$buf .= '</style>';
		$buf .= '<div class="pssDescriptionDiv">';
		if (_PSS_TIMEMSGPRO_LICENCE_==='BASIC')
			$buf .= $this->l('This module allows to schedule HTML texts to be displayed to your visitors in some places of your shop.');
		else
		{
			$buf .= $this->l('This module allows');
			$buf .= '<br /> - '.$this->l('to schedule HTML texts to be displayed to your visitors in some places of your shop, using standard or custom templates, CSS styles').',';
			$buf .= '<br /> - '.$this->l('to schedule custom actions to be triggered upon some events.');
			$buf .= '<br />'.$this->l('You could also define some advanced filters to only display / trigger to some pages (category, product, user group).');
		}
		$buf .= '<br /><b>'.$this->l('Access to this module after installation by the "Preferences" tab.').'</b>';
		$buf .= '</div>';
        $this->description = $buf;
    }

    function install()
    {
		// remove all log files
		PssTimeMsgProUtils::removeLogs();
		PssTimeMsgProUtils::addLog("******************************************************************");
		PssTimeMsgProUtils::addLog("Install module on Prestashop version "._PS_VERSION_);
		PssTimeMsgProUtils::addLog("Module licence : "._PSS_TIMEMSGPRO_LICENCE_);
		
		// check required version - 1.2.x or 1.3.x or 1.4.x
		// @change for Prestashop 1.5 compatibility
		if (!PssTimeMsgProUtils::isPs12x() && !PssTimeMsgProUtils::isPs13x() && !PssTimeMsgProUtils::isPs14x() && !PssTimeMsgProUtils::isPs15x())
		{
			PssTimeMsgProUtils::addLog("Install aborted because Prestashop version don't match criteria");
			return false;
		}
		// normal install
		PssTimeMsgProUtils::addLog("Install starts ...");
		if(!parent::install() || 
			!$this->_installTab() ||
			!$this->_installDb() ||
			!$this->registerHook('header') ||	// mandatory because of data preparation for filters
			!$this->_installConfig()			// to be done at last because deal with file versions in conf
			)
			return false;
		PssTimeMsgProUtils::addLog("Install ends with success");
		return true;
    }
	private function _installTab() {
		PssTimeMsgProUtils::addLog("\tInstallTab starts ...");
		@copy(_PS_MODULE_DIR_ . $this->name . '/logo.gif', _PS_IMG_DIR_ . 't/' . $this->_tabClass . '.gif');
		$tab = new Tab();
		foreach (Language::getLanguages() as $language)
		{
			$iso_code = array_key_exists($language['iso_code'],$this->_moduleNames)?$language['iso_code']:'en';
			$tab->name[$language['id_lang']] = $this->_moduleNames[$iso_code];
		}
		$tab->class_name = $this->_tabClass;
		$tab->module = $this->name;
		PssTimeMsgProUtils::addLog("\t\tbefore get id_parent");
		$tab->id_parent = Tab::getIdFromClassName($this->tabParentName);
		PssTimeMsgProUtils::addLog("\t\tget id_parent : ".$tab->id_parent);
		
		// Change 2012-01-23 : 1.4 : has a bug one time here : no rights were inserted for the admin profile - and save returns false
		// so use add instead of save ...
		//if (!$tab->save())
		//	return false;
		$tab->add();
		// and be sure the rights are set for admin profile
		if ($tab->id)
		{
			PssTimeMsgProUtils::addLog("\t\tok to add rights to admin");
			$query = 'INSERT IGNORE INTO `'._DB_PREFIX_.'access` (`id_profile`, `id_tab`, `view`, `add`, `edit`, `delete`) VALUES (1, '.$tab->id.', 1, 1, 1, 1)';
			Db::getInstance()->Execute($query);
			PssTimeMsgProUtils::addLog("\tInstallTab ends with success");
		}
		else
			PssTimeMsgProUtils::addLog("\t\tUNABLE to add rights on this tab to Admin because created tab had no ID !!");
		
		return true;
	}
	/**
	 * Install the config constants for this module
	 */ 
	private function _installConfig()
	{
		PssTimeMsgProUtils::addLog("\tInstallConfig starts ...");
		foreach (self::$_config as $key => $value) 
			Configuration::updateValue($key, $value);
		PssTimeMsgProUtils::addLog("\tInstallConfig ends with success");
		return true;
	}
	/**
	 * Install the DB tables / datas for this module
	 */ 
	private function _installDb() 
	{
		global $cookie;
		PssTimeMsgProUtils::addLog("\tInstallDB starts ...");
		
		// create this module's classe tables
		// the messages table --------------------------------------------
		$query = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'prestascope_timemsgpro` (
			`id_msg` int(10) unsigned NOT NULL auto_increment,
			`name` varchar(255) NOT NULL,
			`hook_name` varchar(255) NOT NULL,
			`is_message` tinyint(1) NOT NULL DEFAULT \'0\',
			`user_class` varchar(128),
			`template_name` varchar(255) NOT NULL,
			`css_name` varchar(255) NOT NULL,
			`from_date` date,
			`to_date` date,
			`cron_expression` varchar(25),
			`duration` int(10),
			`id_group` int(10),
			`id_category` int(10),
			`id_product` int(10),
			`is_logged` tinyint(1),
			`other_rule` varchar(255),
			`exceptions` varchar(255),
			`active` tinyint(1) NOT NULL default \'1\',
			`order` int(10) NOT NULL default \'0\',
			PRIMARY KEY  (`id_msg`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;';
		Db::getInstance()->Execute($query);		
		PssTimeMsgProUtils::addLog("\t\thave created `" . _DB_PREFIX_ . "prestascope_timemsgpro` table");

		// the messages LANG table --------------------------------------------
		$query = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'prestascope_timemsgpro_lang` (
			`id_msg` int(10) unsigned NOT NULL auto_increment,
			`id_lang` int(10) unsigned NOT NULL,
			`msg` text,
			PRIMARY KEY  (`id_msg`,`id_lang`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;';
		Db::getInstance()->Execute($query);		
		PssTimeMsgProUtils::addLog("\t\thave created `" . _DB_PREFIX_ . "prestascope_timemsgpro_lang` table");

		PssTimeMsgProUtils::addLog("\tInstallDB ends with success");
		return true;
	}
	public function uninstall() 
	{
		PssTimeMsgProUtils::addLog("******************************************************************");
		PssTimeMsgProUtils::addLog("Uninstall starts ...");
		if(!parent::uninstall() || 
			!$this->_uninstallTab() || 
			!$this->_uninstallConfig() || 
			!$this->_uninstallDb() 
			)
			return false;
		PssTimeMsgProUtils::addLog("Uninstall ends with success");
		return true;
	}
	private function _uninstallTab() 
	{
		PssTimeMsgProUtils::addLog("\tUninstallTab starts ...");
		PssTimeMsgProUtils::addLog("\t\tget tab from class name ".$this->_tabClass);
		$idTab = Tab::getIdFromClassName($this->_tabClass);
		PssTimeMsgProUtils::addLog("\t\tget tab : ".$idTab);
		if ($idTab != 0) 
		{
			@unlink(_PS_IMG_DIR_ . 't/' . $this->_tabClass . '.gif');
			$tab = new Tab($idTab);
			PssTimeMsgProUtils::addLog("\t\tbefore delete tab");
			$tab->delete();
			PssTimeMsgProUtils::addLog("\tUninstallTab ends with success");
			return true;
		}
		return false;
	}
	/**
	 * Uninstall the config constants for this module
	 */ 
	private function _uninstallConfig() 
	{
		PssTimeMsgProUtils::addLog("\tUninstallConfig starts ...");
		foreach (self::$_config as $key => $value) 
			Configuration::deleteByName($key);
		PssTimeMsgProUtils::addLog("\tUninstallConfig ends with success");
		return true;
	}
	/**
	 * Uninstall the DB tables / datas for this module
	 */ 
	private function _uninstallDb() 
	{
		PssTimeMsgProUtils::addLog("\tUninstallDB starts ...");
		//create this module's classe tables
		Db::getInstance()->Execute(
			'DROP TABLE `' . _DB_PREFIX_ . 'prestascope_timemsgpro`;'
		);
		Db::getInstance()->Execute(
			'DROP TABLE `' . _DB_PREFIX_ . 'prestascope_timemsgpro_lang`;'
		);
		PssTimeMsgProUtils::addLog("\tUninstallDB ends with success");
		return true;
	}
	
	
	/* *****************************************************************************************
	 *
	 *					HOOKS
	 *
	 * ***************************************************************************************** */
	/**
	 * All hooks use the same generic function to display their messages
	 */
	private function pssHookGeneric($hook_name, $params)
	{
		global $smarty, $cookie, $cart;
		$html= '';
		
		// get valid messages for this display applying restrictions (dates, user group, product, category)
		PssTimeMsgProUtils::addLog("Call for hook ".$hook_name);
//		PssTimeMsgProUtils::addLog(PssTimeMsgProUtils::captureBestVarDump($smarty->get_template_vars()));
//		PssTimeMsgProUtils::addLog(PssTimeMsgProUtils::captureBestVarDump($params));
		$messages = PssTimeMsgPro::getByHook($hook_name, $params, $this->smartyV2, $cookie->id_lang);
		
		// loop through all messages if any
		if (!empty($messages))
		{
			// -------------------------------------------------------------------------------------
			// publish some usually hidden informations like internal variables of cookie
			//
			// cookie common part
			$smarty->assign(array(
				'pss_cookie_id_lang'=>$cookie->__get('id_lang'),
				'pss_cookie_id_currency'=>$cookie->__get('id_currency'),
				'pss_cookie_id_guest'=>$cookie->__get('id_guest')
				));
			// cookie cart part
			if ($cookie->__get('id_cart'))
				$smarty->assign(array(
					'pss_cookie_id_cart'=>$cookie->__get('id_cart')
					));
			// cookie customer part
			if ($cookie->__get('id_customer'))
			{
				$smarty->assign(array(
					'pss_cookie_id_customer'=>$cookie->__get('id_customer'),
					'pss_cookie_customer_lastname'=>$cookie->__get('customer_lastname'),
					'pss_cookie_customer_firstname'=>$cookie->__get('customer_firstname'),
					'pss_cookie_logged'=>$cookie->__get('logged'),
					'pss_cookie_email'=>$cookie->__get('email')
					));
			}
					
			PssTimeMsgProUtils::addLog("Hook ".$hook_name);
			PssTimeMsgProUtils::addLog("\tFound ".count($messages)." messages");
			foreach ($messages as $message)
			{
				PssTimeMsgProUtils::addLog("\tWork with message ".$message['name']." : ");

				// message content
				$smarty->assign(array(
					'pss_msg'=>$message['msg']
					));
					
				// the template file to use (look first in theme folders, then in module folder)
				$tplFile = $this->findTemplate($message['template_name']);
				if ($tplFile)
					$html .= $this->display(__FILE__, $tplFile);
				else
					$html .= $this->l('NO TEMPLATE').' : '.$message['template_name'].'<br />';
			}
		}
		
		return $html;
	}

	/*
	86 hooks from Prestashop 1.4.7 version
	*/
	function hookaddproduct($params){return $this->pssHookGeneric('addproduct', $params);}
	function hookadminCustomers($params){return $this->pssHookGeneric('adminCustomers', $params);}
	function hookadminOrder($params){return $this->pssHookGeneric('adminOrder', $params);}
	function hookAdminStatsModules($params){return $this->pssHookGeneric('AdminStatsModules', $params);}
	function hookafterCreateHtaccess($params){return $this->pssHookGeneric('afterCreateHtaccess', $params);}
	function hookafterDeleteAttribute($params){return $this->pssHookGeneric('afterDeleteAttribute', $params);}
	function hookafterDeleteAttributeGroup($params){return $this->pssHookGeneric('afterDeleteAttributeGroup', $params);}
	function hookafterDeleteFeature($params){return $this->pssHookGeneric('afterDeleteFeature', $params);}
	function hookafterDeleteFeatureValue($params){return $this->pssHookGeneric('afterDeleteFeatureValue', $params);}
	function hookafterSaveAdminMeta($params){return $this->pssHookGeneric('afterSaveAdminMeta', $params);}
	function hookafterSaveAttribute($params){return $this->pssHookGeneric('afterSaveAttribute', $params);}
	function hookafterSaveAttributeGroup($params){return $this->pssHookGeneric('afterSaveAttributeGroup', $params);}
	function hookafterSaveFeature($params){return $this->pssHookGeneric('afterSaveFeature', $params);}
	function hookafterSaveFeatureValue($params){return $this->pssHookGeneric('afterSaveFeatureValue', $params);}
	function hookafterSaveProduct($params){return $this->pssHookGeneric('afterSaveProduct', $params);}
	function hookattributeForm($params){return $this->pssHookGeneric('attributeForm', $params);}
	function hookattributeGroupForm($params){return $this->pssHookGeneric('attributeGroupForm', $params);}
	function hookauthentication($params){return $this->pssHookGeneric('authentication', $params);}
	function hookbackBeforePayment($params){return $this->pssHookGeneric('backBeforePayment', $params);}
	function hookbackOfficeFooter($params){return $this->pssHookGeneric('backOfficeFooter', $params);}
	function hookbackOfficeHeader($params){return $this->pssHookGeneric('backOfficeHeader', $params);}
	function hookbackOfficeHome($params){return $this->pssHookGeneric('backOfficeHome', $params);}
	function hookbackOfficeTop($params){return $this->pssHookGeneric('backOfficeTop', $params);}
	function hookbeforeAuthentication($params){return $this->pssHookGeneric('beforeAuthentication', $params);}
	function hookbeforeCarrier($params){return $this->pssHookGeneric('beforeCarrier', $params);}
	function hookcancelProduct($params){return $this->pssHookGeneric('cancelProduct', $params);}
	function hookcart($params){return $this->pssHookGeneric('cart', $params);}
	function hookcategoryAddition($params){return $this->pssHookGeneric('categoryAddition', $params);}
	function hookcategoryDeletion($params){return $this->pssHookGeneric('categoryDeletion', $params);}
	function hookcategoryUpdate($params){return $this->pssHookGeneric('categoryUpdate', $params);}
	function hookcreateAccount($params){return $this->pssHookGeneric('createAccount', $params);}
	function hookcreateAccountForm($params){return $this->pssHookGeneric('createAccountForm', $params);}
	function hookcreateAccountTop($params){return $this->pssHookGeneric('createAccountTop', $params);}
	function hookcustomerAccount($params){return $this->pssHookGeneric('customerAccount', $params);}
	function hookdeleteproduct($params){return $this->pssHookGeneric('deleteproduct', $params);}
	function hookdeleteProductAttribute($params){return $this->pssHookGeneric('deleteProductAttribute', $params);}
	function hookextraCarrier($params){return $this->pssHookGeneric('extraCarrier', $params);}
	function hookextraLeft($params){return $this->pssHookGeneric('extraLeft', $params);}
	function hookextraProductComparison($params){return $this->pssHookGeneric('extraProductComparison', $params);}
	function hookextraRight($params){return $this->pssHookGeneric('extraRight', $params);}
	function hookfeatureForm($params){return $this->pssHookGeneric('featureForm', $params);}
	function hookfeatureValueForm($params){return $this->pssHookGeneric('featureValueForm', $params);}
	function hookfooter($params){return $this->pssHookGeneric('footer', $params);}
	function hookfrontCanonicalRedirect($params){return $this->pssHookGeneric('frontCanonicalRedirect', $params);}
	function hookGraphEngine($params){return $this->pssHookGeneric('GraphEngine', $params);}
	function hookGridEngine($params){return $this->pssHookGeneric('GridEngine', $params);}
	function hookhome($params){return $this->pssHookGeneric('home', $params);}
	function hookinvoice($params){return $this->pssHookGeneric('invoice', $params);}
	function hookleftColumn($params){return $this->pssHookGeneric('leftColumn', $params);}
	function hookmyAccountBlock($params){return $this->pssHookGeneric('myAccountBlock', $params);}
	function hooknewOrder($params){return $this->pssHookGeneric('newOrder', $params);}
	function hookorderConfirmation($params){return $this->pssHookGeneric('orderConfirmation', $params);}
	function hookorderDetail($params){return $this->pssHookGeneric('orderDetail', $params);}
	function hookorderDetailDisplayed($params){return $this->pssHookGeneric('orderDetailDisplayed', $params);}
	function hookorderReturn($params){return $this->pssHookGeneric('orderReturn', $params);}
	function hookorderSlip($params){return $this->pssHookGeneric('orderSlip', $params);}
	function hookpayment($params){return $this->pssHookGeneric('payment', $params);}
	function hookpaymentCCAdded($params){return $this->pssHookGeneric('paymentCCAdded', $params);}
	function hookpaymentConfirm($params){return $this->pssHookGeneric('paymentConfirm', $params);}
	function hookpaymentReturn($params){return $this->pssHookGeneric('paymentReturn', $params);}
	function hookpaymentTop($params){return $this->pssHookGeneric('paymentTop', $params);}
	function hookPDFInvoice($params){return $this->pssHookGeneric('PDFInvoice', $params);}
	function hookpostProcessAttribute($params){return $this->pssHookGeneric('postProcessAttribute', $params);}
	function hookpostProcessAttributeGroup($params){return $this->pssHookGeneric('postProcessAttributeGroup', $params);}
	function hookpostProcessFeature($params){return $this->pssHookGeneric('postProcessFeature', $params);}
	function hookpostProcessFeatureValue($params){return $this->pssHookGeneric('postProcessFeatureValue', $params);}
	function hookpostUpdateOrderStatus($params){return $this->pssHookGeneric('postUpdateOrderStatus', $params);}
	function hookprocessCarrier($params){return $this->pssHookGeneric('processCarrier', $params);}
	function hookproductActions($params){return $this->pssHookGeneric('productActions', $params);}
	function hookproductfooter($params){return $this->pssHookGeneric('productfooter', $params);}
	function hookproductListAssign($params){return $this->pssHookGeneric('productListAssign', $params);}
	function hookproductOutOfStock($params){return $this->pssHookGeneric('productOutOfStock', $params);}
	function hookproductTab($params){return $this->pssHookGeneric('productTab', $params);}
	function hookproductTabContent($params){return $this->pssHookGeneric('productTabContent', $params);}
	function hookrightColumn($params){return $this->pssHookGeneric('rightColumn', $params);}
	function hooksearch($params){return $this->pssHookGeneric('search', $params);}
	function hookshoppingCart($params){return $this->pssHookGeneric('shoppingCart', $params);}
	function hookshoppingCartExtra($params){return $this->pssHookGeneric('shoppingCartExtra', $params);}
	function hooktop($params){return $this->pssHookGeneric('top', $params);}
	function hookupdateCarrier($params){return $this->pssHookGeneric('updateCarrier', $params);}
	function hookupdateOrderStatus($params){return $this->pssHookGeneric('updateOrderStatus', $params);}
	function hookupdateproduct($params){return $this->pssHookGeneric('updateproduct', $params);}
	function hookupdateProductAttribute($params){return $this->pssHookGeneric('updateProductAttribute', $params);}
	function hookupdateQuantity($params){return $this->pssHookGeneric('updateQuantity', $params);}
	function hookwatermark($params){return $this->pssHookGeneric('watermark', $params);}

	// @change for Prestashop 1.5 compatibility
	/*
	102 hooks from Prestashop 1.5.1 version
	*/
	function hookactionAdminLanguagesControllerStatusBefore($params){return $this->pssHookGeneric('actionAdminLanguagesControllerStatusBefore', $params);}
	function hookactionAdminMetaControllerUpdate_optionsBefore($params){return $this->pssHookGeneric('actionAdminMetaControllerUpdate_optionsBefore', $params);}
	function hookactionAdminMetaSave($params){return $this->pssHookGeneric('actionAdminMetaSave', $params);}
	function hookactionAttributeDelete($params){return $this->pssHookGeneric('actionAttributeDelete', $params);}
	function hookactionAttributeGroupDelete($params){return $this->pssHookGeneric('actionAttributeGroupDelete', $params);}
	function hookactionAttributeGroupSave($params){return $this->pssHookGeneric('actionAttributeGroupSave', $params);}
	function hookactionAttributePostProcess($params){return $this->pssHookGeneric('actionAttributePostProcess', $params);}
	function hookactionAttributeSave($params){return $this->pssHookGeneric('actionAttributeSave', $params);}
	function hookactionAuthentication($params){return $this->pssHookGeneric('actionAuthentication', $params);}
	function hookactionBeforeAuthentication($params){return $this->pssHookGeneric('actionBeforeAuthentication', $params);}
	function hookactionBeforeSubmitAccount($params){return $this->pssHookGeneric('actionBeforeSubmitAccount', $params);}
	function hookactionCarrierProcess($params){return $this->pssHookGeneric('actionCarrierProcess', $params);}
	function hookactionCarrierUpdate($params){return $this->pssHookGeneric('actionCarrierUpdate', $params);}
	function hookactionCartSave($params){return $this->pssHookGeneric('actionCartSave', $params);}
	function hookactionCategoryAdd($params){return $this->pssHookGeneric('actionCategoryAdd', $params);}
	function hookactionCategoryDelete($params){return $this->pssHookGeneric('actionCategoryDelete', $params);}
	function hookactionCategoryUpdate($params){return $this->pssHookGeneric('actionCategoryUpdate', $params);}
	function hookactionCustomerAccountAdd($params){return $this->pssHookGeneric('actionCustomerAccountAdd', $params);}
	function hookactionFeatureDelete($params){return $this->pssHookGeneric('actionFeatureDelete', $params);}
	function hookactionFeatureSave($params){return $this->pssHookGeneric('actionFeatureSave', $params);}
	function hookactionFeatureValueDelete($params){return $this->pssHookGeneric('actionFeatureValueDelete', $params);}
	function hookactionFeatureValueSave($params){return $this->pssHookGeneric('actionFeatureValueSave', $params);}
	function hookactionHtaccessCreate($params){return $this->pssHookGeneric('actionHtaccessCreate', $params);}
	function hookactionObjectCategoryDeleteAfter($params){return $this->pssHookGeneric('actionObjectCategoryDeleteAfter', $params);}
	function hookactionObjectCategoryUpdateAfter($params){return $this->pssHookGeneric('actionObjectCategoryUpdateAfter', $params);}
	function hookactionObjectCmsDeleteAfter($params){return $this->pssHookGeneric('actionObjectCmsDeleteAfter', $params);}
	function hookactionObjectCmsUpdateAfter($params){return $this->pssHookGeneric('actionObjectCmsUpdateAfter', $params);}
	function hookactionObjectManufacturerDeleteAfter($params){return $this->pssHookGeneric('actionObjectManufacturerDeleteAfter', $params);}
	function hookactionObjectManufacturerUpdateAfter($params){return $this->pssHookGeneric('actionObjectManufacturerUpdateAfter', $params);}
	function hookactionObjectProductDeleteAfter($params){return $this->pssHookGeneric('actionObjectProductDeleteAfter', $params);}
	function hookactionObjectProductUpdateAfter($params){return $this->pssHookGeneric('actionObjectProductUpdateAfter', $params);}
	function hookactionObjectSupplierDeleteAfter($params){return $this->pssHookGeneric('actionObjectSupplierDeleteAfter', $params);}
	function hookactionObjectSupplierUpdateAfter($params){return $this->pssHookGeneric('actionObjectSupplierUpdateAfter', $params);}
	function hookactionOrderDetail($params){return $this->pssHookGeneric('actionOrderDetail', $params);}
	function hookactionOrderReturn($params){return $this->pssHookGeneric('actionOrderReturn', $params);}
	function hookactionOrderSlipAdd($params){return $this->pssHookGeneric('actionOrderSlipAdd', $params);}
	function hookactionOrderStatusPostUpdate($params){return $this->pssHookGeneric('actionOrderStatusPostUpdate', $params);}
	function hookactionOrderStatusUpdate($params){return $this->pssHookGeneric('actionOrderStatusUpdate', $params);}
	function hookactionPaymentCCAdd($params){return $this->pssHookGeneric('actionPaymentCCAdd', $params);}
	function hookactionPaymentConfirmation($params){return $this->pssHookGeneric('actionPaymentConfirmation', $params);}
	function hookactionProductAdd($params){return $this->pssHookGeneric('actionProductAdd', $params);}
	function hookactionProductAttributeDelete($params){return $this->pssHookGeneric('actionProductAttributeDelete', $params);}
	function hookactionProductAttributeUpdate($params){return $this->pssHookGeneric('actionProductAttributeUpdate', $params);}
	function hookactionProductCancel($params){return $this->pssHookGeneric('actionProductCancel', $params);}
	function hookactionProductDelete($params){return $this->pssHookGeneric('actionProductDelete', $params);}
	function hookactionProductListOverride($params){return $this->pssHookGeneric('actionProductListOverride', $params);}
	function hookactionProductOutOfStock($params){return $this->pssHookGeneric('actionProductOutOfStock', $params);}
	function hookactionProductSave($params){return $this->pssHookGeneric('actionProductSave', $params);}
	function hookactionProductUpdate($params){return $this->pssHookGeneric('actionProductUpdate', $params);}
	function hookactionSearch($params){return $this->pssHookGeneric('actionSearch', $params);}
	function hookactionShopDataDuplication($params){return $this->pssHookGeneric('actionShopDataDuplication', $params);}
	function hookactionTaxManager($params){return $this->pssHookGeneric('actionTaxManager', $params);}
	function hookactionUpdateQuantity($params){return $this->pssHookGeneric('actionUpdateQuantity', $params);}
	function hookactionValidateOrder($params){return $this->pssHookGeneric('actionValidateOrder', $params);}
	function hookactionWatermark($params){return $this->pssHookGeneric('actionWatermark', $params);}
	function hookdisplayAdminCustomers($params){return $this->pssHookGeneric('displayAdminCustomers', $params);}
	function hookdisplayAdminOrder($params){return $this->pssHookGeneric('displayAdminOrder', $params);}
	function hookdisplayAdminStatsGraphEngine($params){return $this->pssHookGeneric('displayAdminStatsGraphEngine', $params);}
	function hookdisplayAdminStatsGridEngine($params){return $this->pssHookGeneric('displayAdminStatsGridEngine', $params);}
	function hookdisplayAdminStatsModules($params){return $this->pssHookGeneric('displayAdminStatsModules', $params);}
	function hookdisplayAttributeForm($params){return $this->pssHookGeneric('displayAttributeForm', $params);}
	function hookdisplayAttributeGroupForm($params){return $this->pssHookGeneric('displayAttributeGroupForm', $params);}
	function hookdisplayAttributeGroupPostProcess($params){return $this->pssHookGeneric('displayAttributeGroupPostProcess', $params);}
	function hookdisplayBackOfficeFooter($params){return $this->pssHookGeneric('displayBackOfficeFooter', $params);}
	function hookdisplayBackOfficeHeader($params){return $this->pssHookGeneric('displayBackOfficeHeader', $params);}
	function hookdisplayBackOfficeHome($params){return $this->pssHookGeneric('displayBackOfficeHome', $params);}
	function hookdisplayBackOfficeTop($params){return $this->pssHookGeneric('displayBackOfficeTop', $params);}
	function hookdisplayBeforeCarrier($params){return $this->pssHookGeneric('displayBeforeCarrier', $params);}
	function hookdisplayBeforePayment($params){return $this->pssHookGeneric('displayBeforePayment', $params);}
	function hookdisplayCarrierList($params){return $this->pssHookGeneric('displayCarrierList', $params);}
	function hookdisplayCustomerAccount($params){return $this->pssHookGeneric('displayCustomerAccount', $params);}
	function hookdisplayCustomerAccountForm($params){return $this->pssHookGeneric('displayCustomerAccountForm', $params);}
	function hookdisplayCustomerAccountFormTop($params){return $this->pssHookGeneric('displayCustomerAccountFormTop', $params);}
	function hookdisplayFeatureForm($params){return $this->pssHookGeneric('displayFeatureForm', $params);}
	function hookdisplayFeaturePostProcess($params){return $this->pssHookGeneric('displayFeaturePostProcess', $params);}
	function hookdisplayFeatureValueForm($params){return $this->pssHookGeneric('displayFeatureValueForm', $params);}
	function hookdisplayFeatureValuePostProcess($params){return $this->pssHookGeneric('displayFeatureValuePostProcess', $params);}
	function hookdisplayFooter($params){return $this->pssHookGeneric('displayFooter', $params);}
	function hookdisplayFooterProduct($params){return $this->pssHookGeneric('displayFooterProduct', $params);}
	// special call - see below
//	function hookdisplayHeader($params){return $this->pssHookGeneric('displayHeader', $params);}
	function hookdisplayHome($params){return $this->pssHookGeneric('displayHome', $params);}
	function hookdisplayInvoice($params){return $this->pssHookGeneric('displayInvoice', $params);}
	function hookdisplayLeftColumn($params){return $this->pssHookGeneric('displayLeftColumn', $params);}
	function hookdisplayLeftColumnProduct($params){return $this->pssHookGeneric('displayLeftColumnProduct', $params);}
	function hookdisplayMobileTopSiteMap($params){return $this->pssHookGeneric('displayMobileTopSiteMap', $params);}
	function hookdisplayMyAccountBlock($params){return $this->pssHookGeneric('displayMyAccountBlock', $params);}
	function hookdisplayMyAccountBlockfooter($params){return $this->pssHookGeneric('displayMyAccountBlockfooter', $params);}
	function hookdisplayOrderConfirmation($params){return $this->pssHookGeneric('displayOrderConfirmation', $params);}
	function hookdisplayOrderDetail($params){return $this->pssHookGeneric('displayOrderDetail', $params);}
	function hookdisplayPayment($params){return $this->pssHookGeneric('displayPayment', $params);}
	function hookdisplayPaymentReturn($params){return $this->pssHookGeneric('displayPaymentReturn', $params);}
	function hookdisplayPaymentTop($params){return $this->pssHookGeneric('displayPaymentTop', $params);}
	function hookdisplayPDFInvoice($params){return $this->pssHookGeneric('displayPDFInvoice', $params);}
	function hookdisplayProductButtons($params){return $this->pssHookGeneric('displayProductButtons', $params);}
	function hookdisplayProductComparison($params){return $this->pssHookGeneric('displayProductComparison', $params);}
	function hookdisplayProductTab($params){return $this->pssHookGeneric('displayProductTab', $params);}
	function hookdisplayProductTabContent($params){return $this->pssHookGeneric('displayProductTabContent', $params);}
	function hookdisplayRightColumn($params){return $this->pssHookGeneric('displayRightColumn', $params);}
	function hookdisplayRightColumnProduct($params){return $this->pssHookGeneric('displayRightColumnProduct', $params);}
	function hookdisplayShoppingCart($params){return $this->pssHookGeneric('displayShoppingCart', $params);}
	function hookdisplayShoppingCartFooter($params){return $this->pssHookGeneric('displayShoppingCartFooter', $params);}
	function hookdisplayTop($params){return $this->pssHookGeneric('displayTop', $params);}
	function hookDisplayHeader($params)
	{
		return $this->hookHeader($params);
	}
	/**
	 * This hook could not be assigned to any message. 
	 * It's only intented to store some all hooks shared datas using the Smarty's template_vars
	 */
	function hookHeader($params)
	{
		return null;
	}
	
	/* *****************************************************************************************
	 *
	 *					TOOLS FOR HOOKS DISPLAY
	 *
	 * ***************************************************************************************** */
	/**
	 * Find a template looking :
	 * 	1) in www/themes/mytheme/modules/pss_timemsgpro/templates/ custom folder
	 *	2) then in www/modules/pss_timemsgpro/templates/ module folder
	 */
	private function findTemplate($templateName)
	{
		$templateFilename = $templateName.'.tpl';
		$moduleTemplatePath = _PS_MODULE_DIR_.'pss_timemsgpro/templates/'.$templateFilename;
		if (file_exists($moduleTemplatePath))
		{
			PssTimeMsgProUtils::addLog("Look for template ".$templateName. " : find it in Module path");
			return 'templates/'.$templateFilename;
		}
		return null;
	}

}
?>
